# -*- coding: utf-8 -*-
"""
Created on Tue Nov 24 08:40:20 2015

@author: twagner
"""

### imports ###################################################################
import logging

#### imports from #############################################################
from pyqtgraph.dockarea import Dock, DockArea
from pyqtgraph.Qt import QtGui

### local imports #############################################################
from .digital_input import DigitalInput

### logger ####################################################################
logging.getLogger('owis_widget').addHandler(logging.NullHandler())

### global variable ###########################################################
warnStyle = "<span style = 'font-size: 14pt; font-weight: bold'>"

###############################################################################
class MainWindow(QtGui.QMainWindow):
    def __init__(self, hardware):
        super(MainWindow, self).__init__()

        self.hardware = hardware

        self.widgets = []

        self.initUI()
        self.initEvents()

    def initUI(self):
        self.axisLRT = Axis(self.hardware.axisLRT)
        self.axisDRT = Axis(self.hardware.axisDRT)
        self.digitalOutput = DigitalOutput(self.hardware)
        w2 = DigitalInput(self.hardware)
        self.mountPlate = MountPlate(self.hardware)
        self.controlUnit = ControlUnit(self.hardware)
        
        self.widgets = {
            'Axis LRT': self.axisLRT,
            'Axis DRT': self.axisDRT,
            'Digital Output': self.digitalOutput,
            'Digital Input': w2,
            'Mount Plate': self.mountPlate,
            'Control unit': self.controlUnit
        }

        area = DockArea()
        self.setCentralWidget(area)

        docks = []

        for name, widget in self.widgets.items():
            dock = Dock(name, autoOrientation = False)
            dock.addWidget(widget)
            area.addDock(dock)
            docks.append(dock)

        self.statusBar()
        
    def initEvents(self):
        self.digitalOutput.sigDigitalOutput.connect(
            self.mountPlate.doSwitchOutput
        )



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 24 08:40:20 2015

@author: twagner
"""

### imports ###################################################################
import logging

#### imports from #############################################################
from PyQt5.QtWidgets import QHBoxLayout, QPushButton, QWidget

### logger ####################################################################
logging.getLogger('owis_widget').addHandler(logging.NullHandler())

###############################################################################
class MountPlate(QWidget):
    def __init__(self, hardware):
        super(MountPlate, self).__init__()

        self.hardware = hardware
        self.mountPlate = None
        self.wafer = None

        self.initUI()
        self.initEvents()
        
    def initUI(self):
        self.mountPlateButton = QPushButton('mount plate')
        self.mountPlateButton.setCheckable(True)

        self.waferButton = QPushButton('wafer')
        self.waferButton.setCheckable(True)

        self.lightCurtainButton = QPushButton('light curtain')
        self.lightCurtainButton.setCheckable(True)

        layout = QHBoxLayout()
        layout.addWidget(self.mountPlateButton)
        layout.addWidget(self.waferButton)
        layout.addWidget(self.lightCurtainButton)

        self.setLayout(layout)
        
    def initEvents(self):
        self.mountPlateButton.clicked.connect(self.doSwitchMountPlate)
        self.waferButton.clicked.connect(self.doSwitchWafer)
        self.lightCurtainButton.clicked.connect(self.doLightCurtain)


    def doLightCurtain(self):
        if self.lightCurtainButton.isChecked():
            self.hardware.axisLRT.errorState = 4
            self.hardware.axisDRT.errorState = 4
        else:
            self.hardware.axisLRT.errorState = 0
            self.hardware.axisDRT.errorState = 0
            
    def doSwitchMountPlate(self):
        if self.mountPlateButton.isChecked():
            self.hardware.digitalInput[0].value = 1
        else:
            self.hardware.digitalInput[0].value = 0

    def doSwitchOutput(self, pin):
        if pin == 1:
            checked = self.waferButton.isChecked()
            output = self.hardware.digitalOutput[0].value

            if checked:
                if output == 1:
                    self.hardware.digitalInput[0].value = 1
                else:
                    self.hardware.digitalInput[0].value = 0

    def doSwitchWafer(self):
        checked = self.waferButton.isChecked()

        if checked:
            if self.hardware.digitalOutput[0].value == 1:
                self.hardware.digitalInput[0].value = 1
            else:
                self.hardware.digitalInput[0].value = 0
        else:
            if self.hardware.digitalOutput[0].value == 1:
                self.waferButton.setChecked(True)
                self.hardware.digitalInput[0].value = 1
            else:
                self.hardware.digitalInput[0].value = 0



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 24 08:40:20 2015

@author: twagner
"""

### imports ###################################################################
import logging
import numpy as np
import os
import yaml

#### imports from #############################################################
from PyQt5.QtCore import pyqtSignal, QSize, QTimer
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QAction, QGridLayout, QGroupBox, QHBoxLayout
from PyQt5.QtWidgets import QLabel, QLineEdit, QMessageBox
from PyQt5.QtWidgets import QToolButton, QVBoxLayout, QWidget
from PyQt5.QtGui import QDoubleValidator, QIcon, QKeySequence

### relative imports ##########################################################
from .warnings import warnChangeWafer, warnMoveToSampleChangePosition
from .warnings import warnNoVacuum, warnWaferNotSuckedIn
import myowis.owis_timer as owis_timer

### logger ####################################################################
logging.getLogger('owis_widget').addHandler(logging.NullHandler())

###############################################################################
class PS10Widget(QWidget):
    """
    Widget for hardware controls.
    """
    ### signals
    gotoRef = pyqtSignal()
    gotoScan = pyqtSignal()
    sig_ref = pyqtSignal()

    def __init__(self, hardware, configFilename):
        super(PS10Widget, self).__init__()

        self.logger = logging.getLogger('owis_widget')        

        self._scanPosition = 0.
        self.scanPositionSave = 0.

        self.scanPositions = {}
        self.maxPositions = {}
        self.flatOffsets = {}

        # filepath = os.path.dirname(os.path.abspath(__file__))
        filepath = os.getcwd()
        
        fullfile = os.path.join(filepath, configFilename)
        
        with open(fullfile) as data_file:    
            data = yaml.load(data_file)

        lrtData = data['widget']['LRT']
        self.upDist = lrtData['upMove']
        self.upFastDist = lrtData['upMoveFast']
        self.downDist = lrtData['downMove']
        self.downFastDist = lrtData['downMoveFast']

        drtData = data['widget']['DRT']
        self.leftDist = drtData['leftMove']
        self.leftFastDist = data['widget']['DRT']['leftMoveFast']
        self.rightDist = data['widget']['DRT']['rightMove']
        self.rightFastDist = data['widget']['DRT']['rightMoveFast']

        self.iconPath = data['widget']['icons']

        self.msgRef = data['widget']['msgRef']
        
        self.hardware = hardware
        self.hardware.digitalOutput[0].verbosity = 1
        self.hardware.needReference = data['hardware']['needReference']
        self.hardware.needVacuum = data['hardware']['needVacuum']
        self.hardware.offsetMark = data['hardware']['offsetMark']

        positions = data['positions']
        
        for position in positions:
            name = position['name']
            self.scanPositions[name] = position['LRT']
            self.maxPositions[name] = position['LRTmax']
            self.flatOffsets[name] = position['LRTflat']
            
        self.mountPlateType = 'Wafer6'
        self.scanPositionSave = self.scanPositions[self.mountPlateType]
        
        self.initUI()
        self.initEvents()
        
    def initUI(self):
        self.txtPos = QLabel('(%5.1f mm, %5.1f deg.)' % (0.0, 0.0))
        self.txtPos.setMinimumWidth(110)
        self.posTimer = QTimer()

        self.txtMoveR = QLineEdit()
        self.txtMoveTheta = QLineEdit()
        
        buttonSize = QSize(105, 105)
        controlButtonSize = QSize(80, 80)
        iconSize = QSize(46, 46)
        
        owisPath = os.path.dirname(os.path.abspath(__file__))
        iconPath = os.path.join(owisPath, 'icons')
        iconPath = "\\\\fcmfs1\\DATA\\MESSTECH\\FCM_Software\\Icons"
        iconPath = self.iconPath

        # icons http://www.flaticon.com/

        buttons = {
            'Ref': ('&Referenzfahrt', "home-button.png", buttonSize),

            'Down': (
                self.tr('%+0.1f cm' % (self.downDist / 2.0)),
                "down-arrow-of-angle.png",
                controlButtonSize
            ),
                     
            'FastDown': (
                self.tr('%+0.1f cm' % (self.downFastDist / 2.0)),
                "two-down-arrows.png",
                controlButtonSize
            ),
                         
            'FastUp': (
                self.tr('%+0.1f cm' % (self.upFastDist / 2.0)),
                "two-arrows-pointing-up.png",
                controlButtonSize
            ),

            'FastLeft': (
                self.tr(u'%+0.1f°' % self.leftFastDist),
                "rewind-double-arrows-angles.png",
                controlButtonSize
            ),
                         
            'FastRight': (
                self.tr(u'%+0.1f°' % self.rightFastDist),
                "fast-forward-double-right-arrows.png",
                controlButtonSize
            ),
                         
            'Left': (
                self.tr(u'%+0.1f°' % self.leftDist),
                "left-arrow-angle.png",
                controlButtonSize
            ),

            'Right': (
                self.tr(u'%+0.1f°' % self.rightDist),
                "right-arrow-angle.png",
                controlButtonSize
            ),

            'Stop': (
                self.tr('Sto&pp'),
                "cancel-button.png",
                buttonSize
            ),

            'Turn': (
                self.tr(u'180°'),
                "up-curved-arrow.png",
                controlButtonSize
            ),

            'Up': (
                self.tr('%+0.1f cm' % (self.upDist / 2.0)),
                "up-arrow-angle.png",
                controlButtonSize
            ),
        }
        
        for key, btnParams in buttons.items():
            btnText, iconFilename, btnSize = btnParams

            btnName = 'btn' + key
            button = QToolButton()
            button.setMinimumSize(btnSize)
            button.setText(btnText)

            icon = QIcon(os.path.join(iconPath, iconFilename))
            button.setIcon(icon)
            button.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
            button.setIconSize(iconSize)

            setattr(self, btnName, button)
            
        icons = {
            'vacuumIcon': "vacuum-tip.png"
        }

        for key, filename in icons.items():
            icon = QIcon(os.path.join(iconPath, filename))
            setattr(self, key,  icon)

        
        self.txtMoveR.setMaxLength(5)
        self.txtMoveR.setValidator(QDoubleValidator())
        
        self.txtMoveTheta.setMaxLength(5)
        self.txtMoveTheta.setValidator(QDoubleValidator())

        self.lightCurtain = owis_timer.LightCurtain(
            self.hardware, iconSize, buttonSize
        )
        
        self.lightCurtain.vacButton.setIcon(self.vacuumIcon)
        self.lightCurtain.btnRef = self.btnRef
        self.lightCurtain.sig_ref = self.sig_ref
        self.lightCurtain.initDRTPosition = self.hardware.offsetMark
        
        lrtBox = QGroupBox("Lineartisch")
        lrtLayout = QGridLayout()
        lrtLayout.addWidget(self.btnUp, 0, 0)
        lrtLayout.addWidget(self.btnDown, 1 ,0)
        lrtLayout.addWidget(self.btnFastUp, 0, 1)
        lrtLayout.addWidget(self.btnFastDown, 1, 1)
        lrtBox.setLayout(lrtLayout)
        lrtBox.setFixedWidth(190)

        drtBox = QGroupBox("Drehtisch")
        drtLayout = QGridLayout()
        drtLayout.addWidget(self.btnLeft, 0, 0, 2, 2)
        drtLayout.addWidget(self.btnFastLeft, 2, 0, 2, 2)
        drtLayout.addWidget(self.btnTurn, 4, 1, 2, 2)
        drtLayout.addWidget(self.btnRight, 0, 2, 2, 2)
        drtLayout.addWidget(self.btnFastRight, 2, 2, 2, 2)
        drtBox.setLayout(drtLayout)
        drtBox.setFixedWidth(190)

        controlBox = QGroupBox("Owis")
        controlLayout = QGridLayout()
        controlLayout.addWidget(self.btnRef, 2, 0, Qt.AlignCenter)
        controlLayout.addWidget(
            self.lightCurtain.vacButton, 0, 0, Qt.AlignCenter
        )
        controlLayout.addWidget(self.btnStop, 3, 0, Qt.AlignCenter)
        controlLayout.addWidget(self.txtPos, 4, 0, Qt.AlignCenter)
        controlBox.setLayout(controlLayout)

        tableLayout = QVBoxLayout()
        tableLayout.addWidget(lrtBox)
        tableLayout.addWidget(drtBox)
        tableLayout.addStretch()

        owisLayout = QVBoxLayout()
        owisLayout.addWidget(controlBox)
        owisLayout.addStretch()
        
        layout = QHBoxLayout()
        layout.addLayout(tableLayout)
        layout.addLayout(owisLayout)
        layout.addStretch()
        self.setLayout(layout)
        
    def initEvents(self):
        self.btnRef.clicked.connect(self.goReference)
        self.btnUp.clicked.connect(self.doUp)
        self.btnDown.clicked.connect(self.doDown)
        self.btnFastUp.clicked.connect(self.doUpFast)
        self.btnFastDown.clicked.connect(self.doDownFast)
        self.btnLeft.clicked.connect(self.doLeft)
        self.btnFastLeft.clicked.connect(self.doLeftFast)
        self.btnRight.clicked.connect(self.doRight)
        self.btnFastRight.clicked.connect(self.doRightFast)
        self.btnTurn.clicked.connect(self.doTurn)
        self.btnStop.clicked.connect(self.doStop)
        self.txtMoveR.returnPressed.connect(self.doMoveR)
        self.txtMoveTheta.returnPressed.connect(self.doMoveTheta)

        self.posTimer.timeout.connect(self.doUpdatePosition)
        self.posTimer.start(1000)

        actionList = [
            ["go to reference", "Ctrl+R", self.goReference],
            [
                "stop",
                QKeySequence(Qt.Key_Escape),
                self.btnStop.click
            ],    
            ["up", QKeySequence(Qt.Key_Up), self.doUp],
            ["down", QKeySequence(Qt.Key_Down), self.doDown],
            ["left", QKeySequence(Qt.Key_Left), self.doLeft],
            ["right", QKeySequence(Qt.Key_Right), self.doRight],
            ["shiftUp", "Shift+Up", self.doUpFast],
            ["shiftDown", "Shift+Down", self.doDownFast],
            ["shiftLeft", "Shift+Left", self.doLeftFast],
            ["shiftRight", "Shift+Right", self.doRightFast],
            ["turn", "PgDown", self.doTurn],
        ]

        for a in actionList:
            action = QAction(
                a[0],
                self,
                shortcut = QKeySequence(a[1]),
                triggered = a[2])
        
            self.addAction(action)

        
    ### properties
    @property
    def scanPosition(self):
        return self._scanPosition
        
    @scanPosition.setter
    def scanPosition(self, value):
        self._scanPosition = value

    ### methods
    def checkForReference(self, axis):
        if(not self.hardware.checkReference(axis)):
            msgRef = QMessageBox()

            message = (
                u"Keine Referenzposition gefunden. " +
                u"Jetzt eine Referenzfahrt durchführen?"
            )

            msgRef.setText(message)

            msgRef.setStandardButtons(
                QMessageBox.Ok | QMessageBox.Cancel
            )

            ret = msgRef.exec_()
            
            if(ret == QMessageBox.Ok):
                self.goReference()


    def checkForVacuum(self):
        hasVac = self.lightCurtain.vacuum
        
        if(not hasVac and self.hardware.needVacuum):
            warnNoVacuum()
        
        return hasVac


    def checkForSuckedIn(self):
        hasVac = self.checkForVacuum()
        isSuckedIn = self.lightCurtain.suckedIn

        if hasVac:
            if(not isSuckedIn and self.hardware.needVacuum):
                warnWaferNotSuckedIn()
        
        return isSuckedIn


    def goReference(self, warning = True):
        if warning:
            if not self.lightCurtain.vacuum:
                QMessageBox.warning(self, "Referenzfahrt", self.msgRef)

        self.logger.info('moving to reference position ...')
        
        self.hardware.moveRef()
        # self.hardware.moveRefOrg()

        self.lightCurtain.initDRTPosition = self.hardware.reference.drt
        self.sig_ref.emit()


    def goScan(self, position, angle = None):
        self.checkForReference(self.hardware.AXIS_LRT)

        ### move linear        
        self.hardware.moveAbsolute(
            self.hardware.AXIS_LRT,
            position
        )
        
        ### Python3 has no long() method, use int() instead
        try:
            long(0)
        except NameError:
            long = int
        
        ### rotate
        if (
            isinstance(angle, (int, long, float)) and
            not isinstance(angle, bool)
        ):
            self.checkForSuckedIn()
            self.hardware.moveAbsolute(self.hardware.AXIS_DRT, angle)

        self.scanPosition = position

    def doUpdatePosition(self):
        l = self.hardware.getPosition(1)
        alpha = self.hardware.getPosition(2)

        # print('doUpdatePosition')
        
        self.txtPos.setText('(%5.1f mm, %5.1f deg.)' % (l, alpha))


    def doUp(self):
        self.checkForReference(self.hardware.AXIS_LRT)
        self.hardware.moveRelative(self.hardware.AXIS_LRT, self.upDist)


    def doDown(self):
        self.checkForReference(self.hardware.AXIS_LRT)
        self.hardware.moveRelative(self.hardware.AXIS_LRT, self.downDist)


    def doUpFast(self):
        self.checkForReference(self.hardware.AXIS_LRT)
        self.hardware.moveRelative(self.hardware.AXIS_LRT, self.upFastDist)


    def doDownFast(self):
        self.checkForReference(self.hardware.AXIS_LRT)
        self.hardware.moveRelative(self.hardware.AXIS_LRT, self.downFastDist)


    def doLeft(self):
        self.checkForReference(self.hardware.AXIS_DRT)
        self.checkForSuckedIn()
        self.hardware.moveRelative(self.hardware.AXIS_DRT, self.leftDist)

    def doLeftFast(self):
        self.checkForReference(self.hardware.AXIS_DRT)
        self.checkForSuckedIn()
        self.hardware.moveRelative(self.hardware.AXIS_DRT, self.leftFastDist)


    def doRight(self):
        self.checkForReference(self.hardware.AXIS_DRT)
        self.checkForSuckedIn()
        self.hardware.moveRelative(self.hardware.AXIS_DRT, self.rightDist)


    def doRightFast(self):
        self.checkForReference(self.hardware.AXIS_DRT)
        self.checkForSuckedIn()
        self.hardware.moveRelative(self.hardware.AXIS_DRT, self.rightFastDist)


    def doTurn(self):
        self.checkForReference(self.hardware.AXIS_DRT)
        self.checkForSuckedIn()
        self.hardware.moveRelative(self.hardware.AXIS_DRT, 180)

        self.hardware.getCurrentPosition()
        
    def doStop(self):
        self.hardware.axisLRT.stop()
        self.hardware.axisDRT.stop()


    def doMoveR(self):
        self.logger.info("moving relative: %s", self.txtMoveR.text())
        self.hardware.moveRelative(1, float(self.txtMoveR.text()))


    def doMoveTheta(self):
        self.logger.info("Move Theta: %", self.txtMoveTheta.text())
        self.hardware.moveRelative(2, float(self.txtMoveTheta.text()))

    def doDiameter(self, diameter):
        self.logger.info("received diameter: %s", diameter)

        if diameter == '76,2':
            plate = 'Wafer3'
        elif diameter == '100':
            plate = 'Wafer4'
        elif diameter == '150':
            plate = 'Wafer6'
        elif diameter == '200':
            plate = 'Wafer8'
        else:
            self.logger.error("wafer diameter not recognized: %s", diameter)
            plate = 'Wafer6'

        lrt, drt = self.hardware.getCurrentPosition()
            
        pos = self.scanPositions[plate]

        if (np.abs(lrt - pos) > 10.0 and lrt > 0.):
            if warnMoveToSampleChangePosition():
                self.logger.info("going to reference position ...")
                self.goReference(warning = False)
                self.logger.info("reference position reached")
    
                warnChangeWafer(diameter)
    
                self.logger.info("plate for %s mm wafer mounted", diameter)
                

    def restoreScanPosition(self):
        self.scanPosition = self.scanPositionSave



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 24 08:40:20 2015

@author: twagner
"""

### imports ###################################################################
import logging

#### imports from #############################################################
from PyQt5.QtWidgets import QHBoxLayout, QPushButton, QWidget
from PyQt5.QtCore import pyqtSignal

### logger ####################################################################
logging.getLogger('owis_widget').addHandler(logging.NullHandler())

###############################################################################
class DigitalOutput(QWidget):
    sigDigitalOutput = pyqtSignal(int)    

    def __init__(self, hardware):
        super(DigitalOutput, self).__init__()

        self.logger = logging.getLogger('owis_widget')        

        self.hardware = hardware

        self.initUI()
        self.initEvents()
        self.initOutputStates()


    def initOutputStates(self):
        for i in range(5):
            state = self.hardware.digitalOutput[i].value
            
            if state == 0:
                self.button[i].setChecked(False)
            elif state == 1:
                self.button[i].setChecked(True)
        
        
    def initUI(self):
        self.button = []
        layout = QHBoxLayout()
        
        for i in range(5):
            button = QPushButton(str(i))
            button.setCheckable(True)
            self.button.append(button)
            layout.addWidget(button)           

        self.verbosityButton = QPushButton()
        self.verbosityButton.setCheckable(True)

        self.setLayout(layout)


    def initEvents(self):
        for i in range(5):
            self.button[i].clicked.connect(self.doSwitchOutput)        

        self.verbosityButton.clicked.connect(self.doSwitchVerbosity)


    def doSwitchOutput(self):
        button = self.sender()
        iButton = int(button.text())
        pin = iButton + 1

        if button.isChecked():
            state = 1
        else:
            state = 0

        self.logger.info('Switch output pin %i to %i', pin, state)
        self.hardware.digitalOutput[iButton].value = state

        self.logger.info('Emitting digital output pin %i', pin)
        self.sigDigitalOutput.emit(pin)


    def doSwitchVerbosity(self):
        button = self.sender()
        
        if button.isChecked():
            self.hardware.digitalOutput[0].verbosity = 2
        else:
            self.hardware.digitalOutput[0].verbosity = 1



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 24 08:40:20 2015

@author: twagner
"""

### imports ###################################################################
import logging

#### imports from #############################################################
from PyQt5.QtWidgets import QHBoxLayout, QPushButton, QWidget
from PyQt5.QtCore import QTimer

### logger ####################################################################
logging.getLogger('owis_widget').addHandler(logging.NullHandler())

###############################################################################
class Axis(QWidget):
    def __init__(self, axis):
        super(Axis, self).__init__()

        self.axis = axis
        
        self.initUI()
        self.initTimer()
        self.initEvents()
        
    def initUI(self):
        self.errorState = QPushButton()
        self.errorState.setCheckable(True)

        layout = QHBoxLayout()
        layout.addWidget(self.errorState)
        
        self.setLayout(layout)

    def initTimer(self):
        self.timer = QTimer()

        self.timer.start(1000)

    def initEvents(self):
        self.timer.timeout.connect(self.doUpdate)

    def doUpdate(self):
        errorState = self.axis.errorState
        position = self.axis.position
        
        if errorState == 0:
            self.errorState.setChecked(True)
        elif errorState == 4:
            self.errorState.setChecked(False)

        self.errorState.setText(str(position))



# -*- coding: utf-8 -*-
"""
Created on Fri Nov 20 13:51:22 2015

@author: twagner
"""

### imports ###################################################################
import logging
import os
import serial.tools.list_ports
import threading
import time
import yaml

### imports from ##############################################################
from ctypes import c_char_p, c_double, c_int
from inspect import stack

if os.name != 'posix':
    from ctypes import WinDLL

### local imports #############################################################
# import owis_drt

#### logging ##################################################################
logging.getLogger('owis').addHandler(logging.NullHandler())

class Position:
    def __init__(self):
        self.lrt = 0.0
        self.drt = 0.0

###############################################################################
class AnalogInput(object):
    def __init__(self, dll, controller, pin):
        self.logger = logging.getLogger('owis')
        self.dll = dll
        self.controller = controller
        self.pin = pin
        self.verbosity = 1
        
        self._value = None

    @property
    def value(self):
        self._previousValue = self._value
        self._value = self.dll.PS10_GetAnalogInput(self.controller, self.pin)
        error = self.dll.PS10_GetReadError(self.controller)
        
        if error:
            self.logger.error(
                "Error getting analog input from pin %i: %i", self.pin, error
            )
            
        elif self.verbosity == 1:
            if self._value != self._previousValue:
                self.logger.debug(
                    "Analog input pin %i: %i", self.pin, self._value
                )
                
                self._previousValue = self._value

        elif self.verbosity == 2:
            self.logger.debug(
                "Analog input pin %i: %i", self.pin, self._value
            )
               
        return self._value


    @value.setter
    def value(self, value):
        self._value = value
        
        self.dll.analogInput[self.pin - 1] = value

###############################################################################
class Axis(object):
    def __init__(self, dll, controller, axis):
        self.logger = logging.getLogger('owis')
        
        self.axis = axis
        self.controller = controller
        self.dll = dll
        self.offsetMark = 0.       
        self.refMode = 4
        self._targetMode = 0
        self._targetEx = 0
        self.unit = '--'        

    ### properties ###
    #%%
    @property
    def accel(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)
        
    @accel.setter
    def accel(self, value):
        propertyName = stack()[0][3]
        self.setProperty(propertyName, value)

        
    #%%
    @property
    def conversion(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)
       
    @conversion.setter
    def conversion(self, value):
        propertyName = stack()[0][3]
        self.setProperty(propertyName, value)

        
    #%%
    @property
    def errorState(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

    @errorState.setter
    def errorState(self, value):
        if hasattr(self.dll, 'errorState'):
            self.dll.errorState[self.axis - 1] = value        

    #%%
    @property
    def fastRefF(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

    @fastRefF.setter
    def fastRefF(self, value):
        propertyName = stack()[0][3]
        self.setProperty(propertyName, value)
            
    #%%
    @property
    def frequency(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

    @frequency.setter
    def frequency(self, value):
        propertyName = stack()[0][3]
        self.setProperty(propertyName, value)
        
    #%%
    @property
    def limitMax(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

    @limitMax.setter
    def limitMax(self, value):
        propertyName = stack()[0][3]
        self.setProperty(propertyName, value)

        
    #%%
    @property
    def limitMin(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

    @limitMin.setter
    def limitMin(self, value):
        propertyName = stack()[0][3]
        self.setProperty(propertyName, value)

        
    #%%
    @property
    def moveState(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

        
    #%%
    @property
    def posF(self):
        '''
        read positioning velocity of an axis (values in Hz)
        '''
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)
        
    @posF.setter
    def posF(self, value):
        propertyName = stack()[0][3]
        self.setProperty(propertyName, value)


    #%%
    @property
    def position(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

        
    #%%
    @property
    def positionEx(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

        
    #%%
    @property
    def posRange(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

        
    #%%
    @property
    def refDecel(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)
        
    @refDecel.setter
    def refDecel(self, value):
        propertyName = stack()[0][3]
        self.setProperty(propertyName, value)

        
    #%%
    @property
    def refReady(self):
        '''
        read positioning velocity of an axis (values in Hz)
        '''
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

        
    #%%
    @property
    def refSwitch(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)
        

    #%%
    @property
    def refSwitchMode(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)
        

    #%%
    @property
    def slowRefF(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)
        
    @slowRefF.setter
    def slowRefF(self, value):
        propertyName = stack()[0][3]
        self.setProperty(propertyName, value)

        
    #%%
    @property
    def targetEx(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

    @targetEx.setter
    def targetEx(self, position):
        propertyName = stack()[0][3]
        value = c_double(position)
        self.setProperty(propertyName, value)
        

    #%%
    @property
    def targetMode(self):
        propertyName = stack()[0][3]
        return self.getProperty(propertyName)

    @targetMode.setter        
    def targetMode(self, value):
        propertyName = stack()[0][3]
        self.setProperty(propertyName, value)

        
    ### methods ###
    def getProperty(self, propertyName):
        methodName = 'PS10_Get' + propertyName[0].upper() + propertyName[1:]
        getMethod = getattr(self.dll, methodName)
        value = getMethod(self.controller, self.axis)

        self.getReadError(propertyName)

        attributeName = '_' + propertyName
        setattr(self, attributeName, value)
        
        return value

            
    def getReadError(self, propertyName):        
        error = self.dll.PS10_GetReadError(self.controller)

        if error:
            self.logger.error(
                "Error getting axis %s %s: %s",
                self.axis, propertyName, error
            )
            
        return error

        
    def setProperty(self, propertyName, value):        
        methodName = 'PS10_Set' + propertyName[0].upper() + propertyName[1:]
        setMethod = getattr(self.dll, methodName)
        error = setMethod(self.controller, self.axis, value)
        
        if error:
            self.logger.error(
                "Error setting axis %i %s to %s: %i",
                self.axis, propertyName, value, error
            )

        propertyValue = getattr(self, propertyName)
            
        if (value != propertyValue):
            self.logger.error(
                "Axis %i %s value %s differs from %s",
                self.axis, propertyName, value, propertyValue
            )
            
        return error
            
   
    def goRef(self):
        self.logger.debug("Going to axis %i reference position", self.axis)
            
        error = self.dll.PS10_GoRef(self.controller, self.axis, self.refMode)

        if error:
            self.logger.error(
                "Could not go to axis %i reference position in mode %i: %i",
                self.axis, self.refMode, error
            )

        return error


    def goTarget(self):
        error = self.dll.PS10_GoTarget(self.controller, self.axis)

        if error:
            self.logger.error(
                "Could not go to axis %i target: %i",
                self.axis, error
            )

        return error

    #%%
    def motorInit(self):
        '''
        initialize an axis and switch on.

        With this function the axis is completely initialized and afterwards is
        with a current and with active positioning regulator. It must be
        executed after the turning on of the control unit, so that the axis can
        be moved afterwards with the commands REF, PGO, VGO etc. Before the
        following parameters must have been set:
            - limit switch mask,
            - polarity,
            - start regulator parameters.
        '''

        self.logger.debug("Initialising axis %i", self.axis)
        error = self.dll.PS10_MotorInit(self.controller, self.axis)

        if error:
            self.logger.debug(
                "Error initialising axis %i: %i", self.axis, error
            )

        return error


    def motorOn(self):
        self.logger.info("Switching on axis %i", self.axis)
        error = self.dll.PS10_MotorOn(self.controller, self.axis)

        if error:
            self.logger.error(
                "Error switching on axis %i: %i", self.axis, error
            )


    def motorOff(self):
        self.logger.info("Switching off axis %i", self.axis)
        error = self.dll.PS10_MotorOff(self.controller, self.axis)

        if error:
            self.logger.error(
                "Error switching off axis %i: %i", self.axis, error
            )


    def moveAbsolute(self, position):
        self.setTargetAbsolute()

        error = self.dll.PS10_MoveEx(
            self.controller, self.axis, c_double(position), 1
        )
        
        if error:
            self.logger.error(
                "Error moving axis %i to %f: %s", self.axis, position, error
            )
            
        return error

            
    def moveRelative(self, position):
        self.setTargetRelative()

        error = self.dll.PS10_MoveEx(
            self.controller, self.axis, c_double(position), 1
        )
        
        if error:
            self.logger.error(
                "Error moving axis %i a distance of %f: %s",
                self.axis, position, error
            )
            
        return error


    def setTargetAbsolute(self):
        self.targetMode = 1


    def setTargetRelative(self):
        self.targetMode = 0


    def stop(self):
        self.logger.info("Stopping axis %i", self.axis)
        error = self.dll.PS10_Stop(self.controller, self.axis)

        if error:
            self.logger.error("error stopping axis: %s", error)

###############################################################################
class DigitalInput(object):
    def __init__(self, dll, controller, pin):
        self.logger = logging.getLogger('owis')
        self.dll = dll
        self.controller = controller
        self.pin = pin
        self.verbosity = 1
        
        self._previousValue = None
        self._value = None

    @property
    def value(self):
        self._previousValue = self._value
        self._value = self.dll.PS10_GetDigitalInput(self.controller, self.pin)
        error = self.dll.PS10_GetReadError(self.controller)
        
        if error:
            self.logger.error(
                "Error getting digital input from pin %i: %i", self.pin, error
            )
            
        elif self.verbosity == 1:
            if self._value != self._previousValue:
                self.logger.debug(
                    "Digital input pin %i: %i", self.pin, self._value
                )
                
                self._previousValue = self._value

        elif self.verbosity == 2:
            self.logger.debug(
                "Digital input pin %i: %i", self.pin, self._value
            )
               
        return self._value


    @value.setter
    def value(self, value):
        if hasattr(self.dll, 'digitalInput'):
            self._value = value
            self.dll.digitalInput[self.pin - 1] = value
        

###############################################################################
class DigitalOutput(object):
    def __init__(self, dll, controller, pin):
        self.logger = logging.getLogger('owis')

        self.dll = dll
        self.controller = controller
        self.pin = pin
        self._previousValue = 0
        self._value = 0
        self.verbosity = 2

    ### properties
    @property
    def value(self):
        self._previousValue = self._value
        self._value = self.dll.PS10_GetDigitalOutput(self.controller, self.pin)
        error = self.dll.PS10_GetReadError(self.controller)
        
        if error:
            self.logger.error(
                "Error getting digital output %i: %i", self.pin, error
            )

        elif self.verbosity == 1:
            if self._value != self._previousValue:
                self.logger.debug(
                    "Digital output pin %i: %i -> %i",
                    self.pin, self._previousValue, self._value
                )

        elif self.verbosity == 2:
            self.logger.debug(
                "Digital output pin %i: %s", self.pin, self._value
            )
                
        return self._value

    @value.setter
    def value(self, state):
        error = self.dll.PS10_SetDigitalOutput(
            self.controller, self.pin, state
        )
        
        if error:
            self.logger.error(
                "Error setting digital output pin %i: %s", self.pin, error
            )
        else:
            self._value = state
            self.logger.debug("Digital output pin %i: %s", self.pin, state)

###############################################################################
class PwmOutput(object):
    def __init__(self, dll, controller, pin):
        self.logger = logging.getLogger('owis')

        self.dll = dll
        self.controller = controller
        self.pin = pin

    ### properties
    @property
    def value(self):
        self._value = self.dll.PS10_GetPwmOutput(self.controller, self.pin)
        error = self.dll.PS10_GetReadError(self.controller)
        
        if error:
            self.logger.error(
                "Error getting PWM output from pin %i: %i", self.pin, error
            )
        else:
            self.logger.debug("PWM pin %i: %s", self.pin, self._value)
                
        return self._value        

    @value.setter
    def value(self, value):

        error = self.dll.PS10_SetPwmOutput(self.controller, self.pin, value)

        if error:
            self.logger.error(
                "Error setting PWM pin %i to %s: %i", self.pin, value, error)
        else:
            self.logger.debug("PWM pin %i output: %s", self.pin, value)
            
###############################################################################
class PS10(object):
    """
    Hardware controller interface.
    """
    
    MAX_TRY = 3

    def __init__(self, filename):
        self.logger = logging.getLogger('owis')
        self.logger.info("Starting OWIS PS10-32 controller")

        with open(filename) as f:    
            self.paraDict = yaml.load(f)

        self.hardwareParameter = self.paraDict['hardware']

        self.com = None
        self.controller = 1
        self.N_aIn = 4
        self.N_digOut = 5
        self.N_digIn = 4
        self.N_pwm = 2

        '''
            0: success
            -1: function error
            -2: communication error
            -3: syntax error
        '''
        self.error = 0

        self._emergency = 0
        self._emergencyPressed = False
        self._emergencyReleased = False
        self._outputMode = 0
        self._prevEmergency = 0

        self.port = None
        self._slaves = None
        self._serialNumber = 20 * ' '

        self.AXIS_LRT = 1
        self.AXIS_DRT = 2
 
        self.conv_axis = {self.AXIS_LRT : 1e-4, self.AXIS_DRT : 9e-3}

        self.reference = Position()

        self.referenceState = 0
        self.maxReferenceState = 0
        self.needReference = True
        self.needVacuum = True
        self.offsetMark = 0
        self.maxPosition = 1e3

        self.detectPort()

        # self.com = 5
        
        self.baudrate = self.hardwareParameter['baudrate']
        self.dllfile = self.hardwareParameter['dllfile']
        self.logging = self.hardwareParameter['logging']
        self.logfile = self.hardwareParameter['logfile']
        self.timeout = self.hardwareParameter['timeout']

        if self.com is not None:

            self.loadDLL(self.dllfile)
            
            interface = 0
    
            self.connect(
                self.controller, interface, self.com, self.baudrate,
                self.timeout
            )

        if self.error or (self.com is None):
            self.logger.warning("Switching to hardware emulator")
            self.dll = PS10_Emulator()

        self.setCanOpenSlave(101, 1)
        self.getCanOpenSlave()

        if self.logging:
            self.initLogging()
        
        self.initAxes()
        self.initAnalogInput()
        self.initDigitalInput()
        self.initDigitalOutput()

        self.vacuumOutput = self.digitalOutput[0]
        self.vacuumInput = self.digitalInput[0]
        
        self.axes[0].motorInit()
        self.axes[1].motorInit()

        # using the method call leads to strange conversion erros
        self.axes[0].limitMax = 700000
        self.axes[0].limitMin = 0
        
        # set conversion
        # self.axes[0].conversion = 1E-4
        # self.axes[1].conversion = 9E-3

        self.axes[0].offsetMark = 0.
        self.axes[1].offsetMark = 0.


    def initAnalogInput(self):
        self.analogInput = []
        
        for i in range(self.N_aIn):
            analogInput = AnalogInput(self.dll, self.controller, i + 1)
            self.analogInput.append(analogInput)

    def initAxes(self):
        self.axes = []
        
        axesDict = self.paraDict['hardware']['axes']
        
        for a in axesDict:
            index = a['index']
            axisName = "axis" + a['name']
            
            axis = Axis(self.dll, self.controller, index)
            
            setattr(self, axisName, axis)
            self.axes.append(axis)

            for key in a.keys():            
                setattr(axis, key, a[key])

        for a in axesDict:
            axisName = "axis" + a['name']
            axis = getattr(self, axisName)
            
            for key in a.keys():            
                value = getattr(axis, key)

                self.logger.debug('%s %s = %s', axisName, key, value)
        

    def initDigitalInput(self):
        self.digitalInput = []
        
        for i in range(self.N_digIn):
            digInput = DigitalInput(self.dll, self.controller, i + 1)
            self.digitalInput.append(digInput)


    def initDigitalOutput(self):
        self.digitalOutput = []
        
        for i in range(self.N_digOut):
            output = DigitalOutput(self.dll, self.controller, i + 1)
            self.digitalOutput.append(output)

            
    def initLogging(self):
        self.logger.info(
            "Start logging to %s", self.logfile
        )
        
        filename = c_char_p(self.logfile)

        try:
            error = self.dll.PS10_LogFile(self.controller, 1, filename, 0, 1)
        except WindowsError:
            error = 99

        if error:
            self.logger.error(
                "Could not start logging to %s: %i", filename, error
            )


    #%% properties
    @property
    def checkSumMem(self):
        self._checkSum = self.dll.PS10_CheckMem(self.controller)
        error = self.dll.PS10_GetReadError(self.controller)

        if error:
            self.logger.error("Error getting memory check sum: %i", error)
        
        return self._checkSum

    #%%
    @property
    def emergency(self):
        self._prevEmergency = self._emergency
        self._emergency = self.dll.PS10_GetEmergencyInput(self.controller)

        if self._prevEmergency == 0 and self._emergency == 1:
            self._emergencyReleased = True
            
            self.logger.warning('Emergency input released')
        
        elif self._prevEmergency == 1 and self._emergency == 0:
            self._emergencyPressed = True

            self.logger.warning('Emergency input pressed')

        return self._emergency

    #%%
    @property
    def emergencyPressed(self):
        value = self._emergencyPressed
        self._emergencyPressed = False
        return value

    #%%
    @property
    def emergencyReleased(self):
        value = self._emergencyReleased
        self._emergencyReleased = False
        return value

    #%%
    @property
    def outputMode(self):
        self.logger.debug('Getting output mode')

        mode = self.dll.PS10_GetOutputMode(self.controller)
        error = self.dll.PS10_GetReadError(self.controller)
        
        if error:
            self.logger.error("Error getting output mode")
        else:
            self._outputMode = mode
            
            if mode == 0:
                self.logger.debug(
                    "pin 1: digital SPS-output," +
                    "pin 2: digital SPS-output"
                )
            elif mode == 1:
                self.logger.debug(
                    "pin 1: digital SPS-output," +
                    "pin 2: PWM-output"
                )
            elif mode == 2:
                self.logger.debug(
                    "pin 1: PWM-output," +
                    "pin 2: PWM-output")
                
        return mode

    @outputMode.setter
    def outputMode(self, mode):
        self.error = self.dll.PS10_SetOutputMode(self.controller, mode)

        if self.error:
            self.logger.error("Error setting output mode to %s", mode)
        else:
            self._outputMode = self.dll.PS10_GetOutputMode(self.controller)

    #%%
    @property
    def serialNumber(self):
        strBuffer = 20 * ' '
        bufSize = self.dll.PS10_GetSerNumber(1, strBuffer, 20)
        
        self._serialNumber = strBuffer[:bufSize]

        return self._serialNumber


    #%%
    @property
    def slaves(self):
        self._slaves = self.dll.PS10_GetSlaves(self.controller)
        error = self.dll.PS10_GetReadError(self.controller)

        if error:
            self.logger.error("Error getting slaves: %i", error)

        return self._slaves


    #%% methods 
    def connect(self, controller, interface, com, baud, timeout):
        self.logger.debug("Connecting to PS10")
        
        self.error = self.dll.PS10_Connect(
            controller, interface, com, baud, timeout,
            0, 0, 0
        )

        if self.error:
            self.logger.error("Could not connect to PS10: %s", self.error)
        else:
            self.controller = controller
            self.interface = interface
            
            self.logger.debug(
                "Connected to PS10(%s, %s)", controller, interface
            )
            
        return self.error

        
    def detectPort(self):
        ports = list(serial.tools.list_ports.comports())
        self.port = None
        self.com = None

        if len(ports) == 0:
            self.logger.warning("Could not find any serial device!")

        for p in ports:
            if 'STMicroelectronics Virtual COM Port' in p[1]:
                self.logger.debug(p[1])
                self.logger.debug(p[2])
                self.com = int(p[0][3])

                self.port = p                
                
        return self.com


    def loadDLL(self, dllFile):
        self.logger.debug("Loading %s", dllFile)
       
        if os.path.exists(dllFile):
            self.dll = WinDLL(dllFile)
        else:
            self.logger.error("Could not find %s", dllFile)

        self.dll.PS10_GetSerNumber.argtypes = [c_int, c_char_p, c_int]
        self.dll.PS10_GetPositionEx.restype = c_double


    def moveRef(self):
        self.logger.info("Going to reference position")

        error = self.axisLRT.goRef()
        
        if error:
            self.logger.error(
                "Setting axis 1 reference position failed: %i",
                error
            )
            
            # return

        time.sleep(1.0)        
        error = self.axisDRT.goRef()        

        if error:
            self.logger.error(
                "Setting axis 2 reference position failed: %i",
                error
            )
            
            # return

        while self.axisLRT.moveState:
            self.logger.debug("LRT move state: %i", self.axisLRT.moveState)
            time.sleep(0.1)

        while self.axisDRT.moveState:
            self.logger.debug("DRT move state: %i", self.axisDRT.moveState)
            time.sleep(0.1)

        '''
        while not self.axisLRT.refReady:
            time.sleep(0.1)

        while not self.axisDRT.refReady:
            time.sleep(0.1)
        '''

        self.logger.debug("DRT axis ready: %i", self.axisDRT.refReady)

        self.reference.lrt = self.getPosition(self.AXIS_LRT)
        self.reference.drt = self.getPosition(self.AXIS_DRT)

        self.logger.info("Reference position reached")

        self.getCurrentPosition()
        
        self.logger.info(
            "Position: %i, %i", self.axisLRT.position, self.axisDRT.position
        )

        self.logger.info(
            "Position: %i, %i", self.axisLRT.positionEx, self.axisDRT.positionEx
        )

        
    def getCurrentPosition(self):
        lrt = self.getPosition(self.AXIS_LRT)
        drt = self.getPosition(self.AXIS_DRT)

        self.logger.debug(
            "current position: %0.1f %s, %0.1f %s",
            lrt, self.axisLRT.unit,
            drt, self.axisDRT.unit
        )

        return lrt, drt
    
    
    def getPosition(self, axis):
        position = self.dll.PS10_GetPosition(self.controller, axis)
        error = self.dll.PS10_GetReadError(self.controller)

        position *= self.conv_axis[axis]

        if axis == self.AXIS_DRT:
            position += self.offsetMark
        
        if error:
            self.logger.error("Error getting axis %i position", axis)
            
        return position


    def checkReference(self, axis):
        self.referenceState &= (0 << (axis-1))
        '''
        reset reference state for axis:
            - workaround to keep reference state even if the axis was shut down
              due to light curtain obstruction
            - the real reference state will be ignored once it was set at least
              one time
            - this will also turn off software limits
        '''
        self.referenceState |= (self.maxReferenceState << (axis-1)) 

        if self.needReference:
            refState = self.dll.PS10_GetRefReady(self.controller, axis)
            self.maxReferenceState |= (refState << (axis-1))
            # set reference state for axis
            self.referenceState |= (refState << (axis-1))
        else:
            self.maxReferenceState |= (1 << (axis-1))
            self.referenceState |= (1 << (axis-1))

        return self.maxReferenceState & (1 << (axis - 1)) > 0


    def moveRelative(self, axis, position, wait = False):
        self.checkReference(axis)
        
        self.logger.info(
            "Moving axis %s a distance of %s %s",
            axis, position, self.axes[axis-1].unit
        )

        if(
            self.needVacuum and
            axis == self.AXIS_DRT and (
                self.vacuumOutput.value == 0 or
                self.vacuumInput.value == 0
            )
        ):
            # if no vacuum, don't move DRT
            self.logger.warning("Vacuum needed for axis %s", axis)
            return

        if(
            axis == self.AXIS_LRT and
            (self.getPosition(axis) + position) > self.maxPosition
        ):
            # software limit of LRT movement
            self.logger.warning("movement would hit limit ... abort")
            return

        if self.axes[axis-1].moveState > 0:
            # if axis is moving, don't do anything
            self.logger.warning("axis %s is already moving", axis)
            return

        if not self.referenceState & (1 << (axis-1)):
            # if no reference for axis is set, don't move
            self.logger.warning("no reference for axis %s", axis)
            return


        self.axes[axis - 1].moveRelative(position)
            

        if wait:
            while self.axes[axis-1].moveState != 0:
                pass


    def moveAbsolute(self, axis, position, wait = False):
        self.checkReference(axis)

        self.logger.info(
            "moving axis %s a distance of %s %s",
            axis, position, self.axes[axis-1].unit
        )

        if(
            self.needVacuum and
            axis == self.AXIS_DRT and
            (self.vacuumOutput.value == 0 or
            self.vacuumInput.value == 0)
        ): # if no vacuum, don't move DRT
            self.logger.warning("vacuum needed for axis %s", axis)
            return

        if axis == self.AXIS_LRT and position > self.maxPosition:
            # software limit of LRT movement
            self.logger.warning(
                "movement would hit software limit %s abort",
                self.maxPosition
            )
            return

        if self.axes[axis - 1].moveState > 0:
            # if axis is moving, don't do anything
            self.logger.warning("axis %s is already moving", axis)
            return

        if not self.referenceState & (1 << (axis-1)):
            # if no reference for axis is found, don't move
            self.logger.error("no reference for axis %s", axis)
            return

        if axis == self.AXIS_DRT: # if DRT apply position offset
            position -= self.offsetMark

        self.axes[axis - 1].moveAbsolute(position)

        if wait:
            while self.getMoveState(axis) != 0:
                pass


    def setCanOpenSlave(self, slaveNumber, slaveID):
        self.slaveNumber = slaveNumber
        self.slaveID = slaveID

        self.logger.debug(
            "setting CanOpen slave: (" +
            str(slaveNumber) + ", " +  str(slaveID) + ")"
        ) 

        
        error = self.dll.PS10_SetCanOpenSlave(slaveNumber, slaveID)
        
        if error:
            self.logger.error("Error:", error)


    def getCanOpenSlave(self, slaveNumber = None):

        if slaveNumber == None:
            slaveNumber = self.slaveNumber
            
        if slaveNumber > 100:
            self.slaveID = self.dll.PS10_GetCanOpenSlave(slaveNumber)
            error = self.dll.PS10_GetReadError(self.controller)
        else:
            error = -4

        if error:
            self.logger.error("Error getting slaveID:", error)

        return self.slaveID


    def disconnect(self):
        error = self.dll.PS10_Disconnect(self.controller)

        self.logger.debug(
            "Disconnecting from PS10 controller %s", self.controller
        )

        if error:
            self.logger.error("Error disconnecting PS10 controller: %s", error)
            
        return error


    def simpleConnect(self, controller):
        self.logger.debug("Connecting to PS10")
        
        serialNumber = '15050140'
        
        self.error = self.dll.PS10_SimpleConnect(
            controller, serialNumber
        )

        if self.error:
            self.logger.warning("Could not connect to PS10: %s", self.error)
        else:
            self.controller = controller
            
            self.logger.debug(
                "Connected to PS10(%s, %s)", controller, serialNumber
            )
            
        return self.error
###############################################################################
class PS10_Emulator:
    """
    emulation of essential PS10 DLL commands
    """

    def __init__(self):
        self.logger = logging.getLogger('owis')

        self.jobs = []
        self.voltage = 0.0
        self.refMode = 4
        
        self.digitalInput = [1, 1, 1, 1, 1]
        self.digitalOutput = [0, 0, 0, 0, 0]

        self._accel = [0, 0]
        self._conversion = [1e-4, 9e-3]
        self.errorState = [0, 0]
        self._fastRefF = [0, 0]
        self._frequency = [0, 0]
        self._limitMax = [0, 0]
        self._limitMin = [0, 0]
        self._moveState = [0, 0]
        self._outputMode = 0
        self._posF = [0, 0]
        self._position = [0., 0.]
        self._pwmOutput = 0
        self._refReady = [0, 0]
        self._slowRefF = [0, 0]
        self._targetMode = 0


    def goRefWorker(self, axis):
        self.logger.info('moving axis %i to reference', axis)

        time.sleep(1)

        self._moveState[axis - 1] = 0
        self._refReady[axis - 1] = 1
        
        self.logger.info('finished moving axis %i to reference', axis)

    def PS10_CheckMem(self, controller):
        return 0

    def PS10_Connect(
        self, controller, interface, com, baud, handshake, bit1, bit2, bit3
    ):
        return 0
    
    def PS10_Disconnect(self, controller):
        for job in self.jobs:
            job.join()
            
        return 0

        
    def PS10_Stop(self, controller, axis):
        return False

    #%%
    def PS10_GetAccel(self, controller, axis):
        return self._accel[axis - 1]

    def PS10_SetAccel(self, controller, axis, value):
        self._accel[axis - 1] = value
    
    #%%
    def PS10_GetAnalogInput(self, controller, pin):
        value = self.voltage
        return value

    #%%
    def PS10_GetCanOpenSlave(self, slaveNumber):
        return 0

    def PS10_SetCanOpenSlave(self, slaveNumber, slaveID):
        return 0
    
    #%%
    def PS10_GetDigitalInput(self, controller, pin):
        return self.digitalInput[pin - 1]

    #%%
    def PS10_GetDigitalOutput(self, controller, pin):
        return self.digitalOutput[pin - 1]

    def PS10_SetDigitalOutput(self, controller, pin, state):
        self.digitalOutput[pin - 1] = state
        return 0

    #%%
    def PS10_GetEmergencyInput(self, controller):
        return False

    #%%
    def PS10_GetErrorState(self, controller, axis):
        return self.errorState[axis - 1]

    #%%
    def PS10_GetF(self, controller, axis):
        return self._frequency[axis - 1]

    def PS10_SetF(self, controller, axis, value):
        self._frequency[axis - 1] = value

    #%%
    def PS10_GetFastRefF(self, controller, axis):
        return self._fastRefF[axis - 1]

    def PS10_SetFastRefF(self, controller, axis, value):
        self._fastRefF[axis - 1] = value

    #%%%
    def PS10_GetLimitMax(self, controller, axis):
        return self._limitMax[axis - 1]

    def PS10_SetLimitMax(self, controller, axis, value):
        self._limitMax[axis - 1] = value

    #%%
    def PS10_GetLimitMin(self, controller, axis):
        return self._limitMin[axis - 1]

    def PS10_SetLimitMin(self, controller, axis, value):
        self._limitMin[axis - 1] = value

    #%%
    def PS10_GetMoveState(self, controller, axis):
        return self._moveState[axis - 1]
        
    #%%
    def PS10_GetOutputMode(self, controller):
        return self._outputMode

    def PS10_SetOutputMode(self, controller, mode):
        self._outputMode = mode

    #%%
    def PS10_GetPosF(self, controller, axis):
        return self._posF[axis - 1]

    def PS10_SetPosF(self, controller, axis, value):
        self._posF[axis - 1] = value

    #%%
    def PS10_GetPosition(self, controller, axis):
        position = self._position[axis - 1]
        return position

    #%%
    def PS10_GetPositionEx(self, controller, axis):
        return 0

    #%%
    def PS10_GetPwmOutput(self, controller, pin):
        return self._pwmOutput

    def PS10_SetPwmOutput(self, controller, pin, value):
        self._pwmOutput = value
        return 0
    
    #%%
    def PS10_GetReadError(self, controller):
        error = False
        return error

    #%%
    def PS10_GetRefDecel(self, controller, axis):
        return 0
        
    def PS10_SetRefDecel(self, controller, axis, value):
        self._refDecel = value

        
    def PS10_GetRefReady(self, controller, axis):
        return self._refReady[axis - 1]

    def PS10_GetRefSwitch(self, controller, axis):
        return 0

    def PS10_GetRefSwitchMode(self, controller, axis):
        return 0

    #%%
    def PS10_GetSlaves(self, controller):
        return 0

    #%%
    def PS10_GetSlowRefF(self, controller, axis):
        return self._slowRefF[axis - 1]

    def PS10_SetSlowRefF(self, controller, axis, value):
        self._slowRefF[axis - 1] = value

    #%%
    def PS10_GoRef(self, controller, axis, mode):

        self._moveState[axis - 1] = 3

        t = threading.Thread(
            target = self.goRefWorker, args = (axis,)
        )

        t.start()

        return 0


    def PS10_MotorInit(self, controller, axis):
        return 0

    def PS10_MotorOff(self, controller, axis):
        pass


    def PS10_MoveEx(self, controller, axis, position, flag):
        if self._targetMode == 0:
            self._position[axis - 1] += position.value
        elif self._targetMode == 1:
            self._position[axis - 1] = position.value
            
        return 0


    def PS10_SetCoversion(self, controller, axis, vaule):
        return 0

        
    def PS10_GetTargetMode(self, controller, axis):
        return self._targetMode
        
    def PS10_SetTargetMode(self, controller, axis, value):
        self._targetMode = value
        return 0

###############################################################################
def oldTests(self):



    ### SPS outputs

    # set both outputs to SPS
    # ps10.outputMode = 0
    
    #ps10.motorOff(1)
    #ps10.motorOff(2)
    ps10.getRefSwitch(1, True)
    ps10.getRefSwitch(2, True)

    ps10.getRefSwitchMode(1, True)
    ps10.getRefSwitchMode(2, True)

    logger.info(
        "Fast Ref DRT %s", ps10.dll.PS10_GetFastRefF(ps10.controller, 2)
    )
    
    logger.info(
        "Slow Ref DRT %s", ps10.dll.PS10_GetSlowRefF(ps10.controller, 2)
    )

    logger.info(
        "Fast Ref LRT %s", ps10.dll.PS10_GetFastRefF(ps10.controller, 1)
    )
    
    logger.info(
        "Slow Ref LRT %s", ps10.dll.PS10_GetSlowRefF(ps10.controller, 1)
    )
    
    logger.info(
        "Speed LRT %s", ps10.dll.PS10_GetF(ps10.controller, 1)
    )
    logger.info(
        "Accel LRT %s", ps10.dll.PS10_GetAccel(ps10.controller, 1)
    )

    logger.info("Speed DRT %s", ps10.dll.PS10_GetF(ps10.controller, 1))
    logger.info("Accel DRT %s", ps10.dll.PS10_GetAccel(ps10.controller, 1))

#    ps10.dll.PS10_SetLimitControl(ps10.controller,1,0b11)
#    ps10.dll.PS10_SetLimitControl(ps10.controller,2,0b11)
        
    logger.info("POS LRT %s", ps10.getPosition(1))
#    timestamp(ps10.getPosition(2))
    ps10.moveRef(True)
    logger.info("POS LRT %s", ps10.getPosition(1))
#    timestamp(ps10.getPosition(2))
    ps10.moveRelative(1, 5)
#    ps10.moveRelative(2,90)
    logger.info("POS LRT %s", ps10.getPosition(1))
    #timestamp(ps10.getPosition(2))
    # timestamp(bin(ps10.dll.PS10_GetLimitSwitch(ps10.controller,1)))
    #timestamp(bin(ps10.dll.PS10_GetLimitSwitch(ps10.controller,2)))
    
    logger.info(ps10.dll.PS10_SetLimitMax(ps10.controller, 1, 700000))
    logger.info(ps10.dll.PS10_SetLimitMin(ps10.controller, 1, -10))

    # timestamp(ps10.dll.PS10_GetLimitMin(ps10.controller,1))
    #timestamp(ps10.dll.PS10_GetLimitMin(ps10.controller,2))
    # timestamp(ps10.dll.PS10_GetLimitMax(ps10.controller,1))
    
    # timestamp(ps10.dll.PS10_SetLimitMax(ps10.controller,2,40000))
    # timestamp(ps10.dll.PS10_SetLimitMin(ps10.controller,2,-40000))

    
    # timestamp(ps10.dll.PS10_GetLimitMin(ps10.controller,1))
    #timestamp(ps10.dll.PS10_GetLimitMin(ps10.controller,2))
    # timestamp(ps10.dll.PS10_GetLimitMax(ps10.controller,1))
    #timestamp(ps10.dll.PS10_GetLimitMax(ps10.controller,2))


    # timestamp(ps10.getPosRange(1))
    # timestamp(ps10.getPosRange(2))
    
    # timestamp(ps10.dll.PS10_GetPosition(ps10.controller,1))
    #timestamp(ps10.dll.PS10_GetPosition(ps10.controller,2))
    
    ps10.moveAbsolute(1, 65, True)
    #ps10.moveAbsolute(2,355,True)
    #ps10.moveRef(True)
    ps10.moveAbsolute(1, 30, True)
    ps10.moveAbsolute(2, 160, True)
    #ps10.moveAbsolute(2,365,True)
    logger.info("POS LRT %s", ps10.getPosition(1))
    logger.info("POS DRT %s", ps10.getPosition(2))
    #time.sleep(5)
    # timestamp(1.0*ps10.dll.PS10_GetPosition(ps10.controller,1)/1e4)
    # timestamp(1.0*ps10.dll.PS10_GetPosition(ps10.controller,2)*360/4e4)
    
    #ps10.moveAbsolute(2,0)
    
    #time.sleep(20)
    #timestamp(1.0*ps10.dll.PS10_GetPosition(ps10.controller,2)*360/4e4)

    ps10.motorOff(1)
    ps10.motorOff(2)
    
    '''
    ps10.getRefSwitch(1, True)
    ps10.getRefSwitch(2, True)

    ps10.getRefSwitchMode(1, True)
    ps10.getRefSwitchMode(2, True)

    ps10.getPositionEx(1)
    ps10.getPositionEx(2)

    ps10.moveRef()

    ps10.moveRelative(1, 40.)
    '''



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 24 08:40:20 2015

@author: twagner
"""

### imports ###################################################################
import logging

#### imports from #############################################################
from PyQt5.QtWidgets import QHBoxLayout, QPushButton, QWidget
from PyQt5.QtCore import QTimer

### logger ####################################################################
logging.getLogger('owis_widget').addHandler(logging.NullHandler())

###############################################################################
class DigitalInput(QWidget):
    def __init__(self, hardware):
        super(DigitalInput, self).__init__()

        self.counter = 0
        self.hardware = hardware
        self.logger = logging.getLogger('owis_widget')        
        self.refreshRate = 1.

        self.initUI()
        self.initTimer()
        self.initEvents()
        
    def initUI(self):
        self.button = []
        layout = QHBoxLayout()
        
        for i in range(4):
            button = QPushButton(str(i))
            button.setCheckable(True)
            self.button.append(button)
            layout.addWidget(button)           

        self.setLayout(layout)

    def initEvents(self):
        self.timer.timeout.connect(self.doUpdate)

    def initTimer(self):
        self.timer = QTimer()
        self.logger.info("starting timer")

        refreshTime = 1000 / self.refreshRate
        self.timer.start(refreshTime)

    def doUpdate(self):
        digitalInput = self.hardware.digitalInput[0].value

        if digitalInput == 1:
            self.button[0].setChecked(True)
        else:
            self.button[0].setChecked(False)

    def stopTimer(self):
        self.logger.info("stoping timer")
        self.timer.stop()



# -*- coding: utf-8 -*-
"""
Created on Mon Feb 15 08:30:13 2016

@author: twagner
"""

### imports ###################################################################
import logging
import qtpy
import qtpy.QtGui as QtGui

if qtpy.PYQT4:
    import PyQt4.QtCore as QtCore
elif qtpy.PYQT5:
    import PyQt5.QtCore as QtCore
else:    
    import qtpy.QtCore as QtCore
    
import qtpy.QtWidgets as QtWidgets

###############################################################################
from qtpy.QtCore import QTimer

# from PyQt4.QtCore import SIGNAL 

# from eval_aux import timestamp

logging.getLogger('owis_timer').addHandler(logging.NullHandler())

###############################################################################
class LightCurtain(QtWidgets.QWidget):
    """
    Originally the light curtain watcher.
    Now performing all background checks of controller state changes.
    Controls vacuum as well.
    """

    sig_ref = QtCore.pyqtSignal()

    def __init__(
        self, hardware = None, iconSize = QtCore.QSize(54, 54),
        buttonSize = QtCore.QSize(110,110)
    ):
        super(LightCurtain, self).__init__()

        self.logger = logging.getLogger('owis_timer')

        self.refreshRate = 10 # 1/s

        self.offMsg = self.tr(
                'Zur Entnahmeposition \n fahren \n &Vakuum ausschalten')
        
        self.onMsg = self.tr('&Vakuum einschalten')

        self.hardware = hardware
        self.hardware.digitalInput[0].verbosity = 1
        self.hardware.digitalOutput[0].verbosity = 1

        self.initDRTPosition = self.hardware.reference.drt

        self.waitForVacuum = False
        self.waitForReference = False

        self.oldReferenceState = -1

        self.iconSize = iconSize
        self.buttonSize = buttonSize

        # dummy until we get the real button from super class
        self.btnRef = QtWidgets.QToolButton()

        self._suckedIn = False
        self._vacuum = False

        self.initUI()
        self.initTimer()
        self.initEvents()

        
    def initUI(self):
        
        self.vacButton = QtWidgets.QToolButton()
        self.vacButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        self.vacButton.setText(self.onMsg)
        self.vacButton.setIcon(QtGui.QIcon('icons/vacuum-tip.png'))
        self.vacButton.setIconSize(self.iconSize)
        self.vacButton.setMinimumSize(self.buttonSize)
        
        if self.vacuum:
            self.vacuum = True
        else:
            self.vacuum = False

        layout = QtWidgets.QHBoxLayout()
        layout.addWidget(self.vacButton)

        self.setLayout(layout)


    def initEvents(self):
        self.timer.timeout.connect(self.update)
        # self.connect(self.timer, SIGNAL("timeout()"), self.update)
        self.vacButton.clicked.connect(self.doSwitchVac)


    def initTimer(self):
        self.timer = QTimer()
        self.logger.info("Starting timer")
        refreshTime = 1000 / self.refreshRate
        self.timer.start(refreshTime)

    #%%
    @property
    def suckedIn(self):
        if self.hardware.vacuumInput.value == 1:
            self._suckedIn = True
        else:
            self._suckedIn = False
            
        return self._suckedIn
    
    @suckedIn.setter
    def suckedIn(self, value):
        pass
        
    #%%
    @property
    def vacuum(self):
        if self.hardware.vacuumOutput.value == 1:
            self._vacuum = True
        else:
            self._vacuum = False
            
        return self._vacuum

    @vacuum.setter
    def vacuum(self, value):
        if value:
            self.hardware.vacuumOutput.value =  1

            self.initDRTPosition = self.hardware.getPosition(
                self.hardware.AXIS_DRT
            )
            
            self.vacButton.setText(self.offMsg)
    
            self.logger.info(
                "Switch on vacuum, current DRT position: %d)",
                self.initDRTPosition
            )
        else:
            self.hardware.vacuumOutput.value = 0
            self.vacButton.setStyleSheet('background-color: none')
            self.vacButton.setText(self.onMsg)
            self.logger.info("Switch off vacuum")
            
    ### methods
    def doSwitchVac(self):
        if self.vacuum == False:
            self.vacuum = True

        else:
            pos = self.hardware.getPosition(self.hardware.AXIS_DRT)

            if(pos != self.initDRTPosition):
                self.logger.info("DRT is rotated. moving back: %s", pos)

                self.hardware.moveAbsolute(
                    self.hardware.AXIS_DRT, self.initDRTPosition
                )

                self.waitForVacuum = True

            else:
                self.vacuum = False

    #### timer control
    def stop(self):
        self.logger.info("Stoping timer")
        self.timer.stop()


    def update(self):
        # %% reference control
        self.hardware.checkReference(self.hardware.AXIS_DRT)
        self.hardware.checkReference(self.hardware.AXIS_LRT)
        
        if self.oldReferenceState != self.hardware.referenceState:
            self.sig_ref.emit()
            
            if self.hardware.maxReferenceState != 3:
                self.btnRef.setStyleSheet('background-color: red')
            else:
                self.btnRef.setStyleSheet('background-color: none')

            self.oldReferenceState = self.hardware.referenceState

        # %% vacuum control
        pos = self.hardware.getPosition(self.hardware.AXIS_DRT)
        
        if (self.waitForVacuum and pos == self.initDRTPosition):
            self.vacuum = False
            self.waitForVacuum = False

        if self.vacuum:
            if self.suckedIn == True:
                self.vacButton.setStyleSheet('background-color: green')
            else:
                self.vacButton.setStyleSheet('background-color: orange')
        else:
            self.vacButton.setStyleSheet('background-color: none')

        # watch emergency stop
        emergency = self.hardware.emergency
        
        if self.hardware.emergencyReleased:        
            self.hardware.axisLRT.motorInit()
            self.hardware.axisDRT.motorInit()

# %% main() ###################################################################    
if __name__ == "__main__":
    from owis import PS10

    # %% setup logger
    logging.basicConfig(
        format = '%(asctime)s %(name)11s %(levelname)7s %(message)s',
        level = logging.DEBUG,
        datefmt = '%Y-%m-%d %H:%M:%S'
    )

    console = logging.StreamHandler()
    fileHandler = logging.FileHandler('..\\log\\owis_timer.log', 'w')    
    
    logger = logging.getLogger("owis_timer")
    logger.addHandler(fileHandler)

    # %% init Qt application
    app = QtWidgets.QApplication([])
    
    # %% init OWIS controller
    ps10 = PS10('..\\config\\company_owis.cfg')
    
    # %% light curtain / vacuum control widget    
    lightCurtain = LightCurtain(ps10)
    lightCurtain.show()
    
    # %% start QtWidget
    app.exec_()



### imports ###################################################################
import logging
import time
import unittest

### local imports #############################################################
import owis

###############################################################################
class OwisTest(unittest.TestCase):

    logger = logging.getLogger('owis_test')

    ps10 = owis.PS10('..\\config\\company_owis.cfg')
    
    analogInputs = [0, 0, 0, 0]
    com = 3
    comDesc = 'STMicroelectronics Virtual COM Port (COM3)'
    comStr = 'COM3'
    controller = 1
    digitalInputs = [0, 1, 1, 1]
    digitalOutputOff = [0, 0, 0, 0, 0]
    digitalOutputOn = [1, 1, 1, 1, 1]
    # usb = 'USB VID:PID=0483:5740 SNR=DEMO_1'
    usb = 'USB VID:PID=0483:5740 SER=DEMO_1 LOCATION=1-5'


    def setUp(self):
        pass

    '''
    def test_00_getBoardVersion(self):
        version = c_char_p()

        self.ps10.dll.PS10_GetBoardVersion.argtypes = [
            c_long, c_char_p, c_long
        ]
        
        length = self.ps10.dll.PS10_GetBoardVersion(1, version, 25)
        print(length, version)
    '''

    @unittest.skip('not working yet')
    def test_00_checkMem(self):
        checkSum = self.ps10.checkSumMem
        print(checkSum)
        

    def test_01_detectPort(self):
        self.ps10.detectPort()
        port = self.ps10.port

        if port is not None:        
            self.assertEqual(port[0], self.comStr)
            self.assertEqual(port[1], self.comDesc)
            self.assertEqual(port[2], self.usb)


    def test_02_motorInit(self):
        for i in range(2):
            error = self.ps10.axes[i].motorInit()
            
            self.assertEqual(error, 0)


    def test_03_connect(self):
        self.assertEqual(self.ps10.disconnect(), 0)
        
        interface = 0
        baud = 9600
        handshake = 0

        error = self.ps10.connect(
            self.controller, interface, self.com, baud, handshake
        )        
        
        self.assertEqual(error, 0)

    #%% test inputs
    def test_04_getAnalogInput(self):
        analogInputs = []
        
        for i in range(4):
            value = self.ps10.analogInput[i].value

            analogInputs.append(value)

        self.assertEqual(analogInputs, self.analogInputs)


    def test_05_getDigitalInput(self):
        digitalInputs = []

        for i in range(4):
            value = self.ps10.digitalInput[i].value

            digitalInputs.append(value)
            
        self.assertEqual(digitalInputs, self.digitalInputs)

    #%% test outputs
    def test_06_outputMode(self):
        self.ps10.outputMode = 2
        self.assertEqual(self.ps10.outputMode, 2)

        self.ps10.outputMode = 1
        self.assertEqual(self.ps10.outputMode, 1)

        self.ps10.outputMode = 0
        self.assertEqual(self.ps10.outputMode, 0)

    #%% test digital output
    def test_07_digitalOutputAllOn(self):
        outputList = []

        for i in range(5):
            self.ps10.digitalOutput[i].value = 1
        
        for i in range(5):
            value = self.ps10.digitalOutput[i].value
            outputList.append(value)

        self.assertEqual(outputList, self.digitalOutputOn)

            
    def test_08_getDigitalOutputAllOff(self):
        outputList = []

        for i in range(5):
            self.ps10.digitalOutput[i].value = 0
        
        for i in range(5):
            value = self.ps10.digitalOutput[i].value
            outputList.append(value)

        self.assertEqual(outputList, self.digitalOutputOff)


    def test_09_axis(self):
        
        self.ps10.axes[0].f = 20000
        self.ps10.axes[0].limitMax = 700000
        self.ps10.axes[0].limitMin = 0
        self.ps10.axes[0].fastRefF = -50000
        self.ps10.axes[0].slowRefF = 2500
        self.ps10.axes[0].posF = 75000
        self.ps10.axes[0].accel = 250000
        # self.ps10.axes[0].conversion = 1e-4

        
        self.assertEqual(self.ps10.axes[0].f, 20000)
        self.assertEqual(self.ps10.axes[0].limitMax, 700000)
        self.assertEqual(self.ps10.axes[0].limitMin, 0)
        self.assertEqual(self.ps10.axes[0].fastRefF, -50000)
        self.assertEqual(self.ps10.axes[0].slowRefF, 2500)
        self.assertEqual(self.ps10.axes[0].posF, 75000)
        self.assertEqual(self.ps10.axes[0].accel, 250000)
        # self.assertEqual(self.ps10.axes[0].conversion, 1e-4)

    def test_10_axis(self):
        self.ps10.axes[1].f = 20000
        # self.ps10.axes[1].limitMax = 7000000
        # self.ps10.axes[1].limitMax = 700000
        # self.ps10.axes[1].limitMin = 0
        self.ps10.axes[1].fastRefF = -5000
        # self.ps10.axes[1].slowRefF = 2500
        # self.ps10.axes[1].posF = 75000
        # self.ps10.axes[1].accel = 250000
        # self.ps10.axes[0].conversion = 1e-4
        
        self.assertEqual(self.ps10.axes[1].f, 20000)
        # self.assertEqual(self.ps10.axes[1].limitMax, 0)
        # self.assertEqual(self.ps10.axes[1].limitMin, 0)
        # self.assertEqual(self.ps10.axes[1].fastRefF, -5000)
        # self.assertEqual(self.ps10.axes[1].slowRefF, 2500)
        # self.assertEqual(self.ps10.axes[1].posF, 75000)
        # self.assertEqual(self.ps10.axes[1].accel, 250000)

    def test_15_goRef(self):
        for iAxis in range(2):
            refReady = self.ps10.axes[iAxis].refReady
            self.assertEqual(refReady, 0)

            error = self.ps10.axes[iAxis].goRef()
            self.assertEqual(error, 0)

            # iCounter = 0
        
            while (
                # iCounter < 100 and
                self.ps10.axes[iAxis].moveState == 3
            ):
                
                time.sleep(0.1)
                # iCounter += 1
                

            refReady = self.ps10.axes[iAxis].refReady
            self.assertEqual(refReady, 1)
        

    def test_16_getPosition(self):
        lrt = self.ps10.getPosition(self.ps10.AXIS_LRT)
        drt = self.ps10.getPosition(self.ps10.AXIS_DRT)

        self.assertEqual(lrt, 0.)
        self.assertEqual(drt, 0.)


    def test_99_disconnect(self):
        self.assertEqual(self.ps10.disconnect(), 0)


    def tearDown(self):
        pass


###############################################################################
if __name__ == '__main__':
    
    ### configuration
    loggingLevel = logging.DEBUG
    # loggingLevel = logging.INFO

    ### set logging
    logging.basicConfig(level = loggingLevel)
    logger = logging.getLogger()

    ### run unit test
    unittest.main()


        



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 24 08:40:20 2015

@author: twagner
"""

### imports ###################################################################
import logging

#### imports from #############################################################
from PyQt5.QtWidgets import QMessageBox

### logger ####################################################################
logging.getLogger('owis_widget').addHandler(logging.NullHandler())

### global variable ###########################################################
warnStyle = "<span style = 'font-size: 14pt; font-weight: bold'>"

###############################################################################
def warnMoveToSampleChangePosition():
    message = (
        warnStyle + 
        u"Der Waferdurchmesser hat sich geändert." + "\n" +
        "Fahre zu Beladeposition."
    )

    title = "Beladeposition anfahren"

    msgBox = QMessageBox()

    yesButton = msgBox.addButton("Ja", QMessageBox.YesRole)
    noButton = msgBox.addButton("Nein", QMessageBox.NoRole)
    
    msgBox.setWindowTitle(title)
    msgBox.setText(message)
    msgBox.setDefaultButton(noButton)
    
    msgBox.exec_()

    result = (msgBox.clickedButton() == yesButton)

    return result

        
def warnNoVacuum():
    msgBox = QMessageBox()
    msgBox.setWindowTitle("Kein Vakuum")

    message = (u"Drehung ist ohne angelegtes Vakuum nicht möglich.")
    
    msgBox.setText(message)
    
    msgBox.exec_()
    

def warnChangeWafer(diameter):
    msgBox = QMessageBox()

    message = (
        warnStyle +
        u"Auflageplatte für " + diameter + " mm Wafer jetzt wechseln!"
    )

    msgBox.setText(message)
    msgBox.setStandardButtons(QMessageBox.Ok)
    msgBox.exec_()
    

def warnWaferNotSuckedIn():
    msgBox = QMessageBox()
    msgBox.setWindowTitle("Kein Vakuum")

    message = (
            u"""Der Wafer ist für eine Drehung nicht genügend angesaugt
                oder es liegt kein Wafer auf.""")
    
    msgBox.setText(message)
    
    msgBox.exec_()



# -*- coding: utf-8 -*-
"""
Created on Tue Nov 24 08:40:20 2015

@author: twagner
"""

### imports ###################################################################
import logging

#### imports from #############################################################
from PyQt5.QtWidgets import QHBoxLayout, QPushButton, QWidget
from PyQt5.QtCore import QTimer

### logger ####################################################################
logging.getLogger('owis_widget').addHandler(logging.NullHandler())

###############################################################################
class ControlUnit(QWidget):
    def __init__(self, hardware):
        super(ControlUnit, self).__init__()

        self.logger = logging.getLogger('owis_widget')        

        self.hardware = hardware

        self.refreshRate = 1
        
        self.initUI()
        self.initTimer()
        self.initEvents()

    def initEvents(self):
        self.timer.timeout.connect(self.doUpdate)

    def initUI(self):
        self.button = QPushButton()        
        self.button.setCheckable(True)

        layout = QHBoxLayout()
        layout.addWidget(self.button)
        
        self.setLayout(layout)

    def initTimer(self):
        self.timer = QTimer()
        self.logger.info("starting timer")

        refreshTime = 1000 / self.refreshRate
        self.timer.start(refreshTime)

    def doUpdate(self):
        emergency = self.hardware.emergency
        
        if emergency:
            self.button.setChecked(True)
        else:
            self.button.setChecked(False)


# -*- coding: utf-8 -*-

import logging
import os

from setuptools import find_packages, setup

AUTHOR = 'hirschbeutel'
AUTHOR_EMAIL = 'hirschbeutel@gmail.com'

DESCRIPTION = 'description'
LONG_DESCRIPTION = 'long description'
PACKAGE_NAME = 'owis-ps10'
URL = 'https://bitbucket.org/hirschbeutel/'

def read_package_variable(key, filename='__init__.py'):
    """Read the value of a variable from the package without importing."""
    module_path = os.path.join(PACKAGE_NAME, filename)

    with open(module_path) as module:
        for line in module:
            parts = line.strip().split(' ', 2)

            if parts[:-1] == [key, '=']:
                return parts[-1].strip("'")

    logging.warning("'%s' not found in '%s'", key, module_path)
    return None

setup(
        author=AUTHOR,
        author_email=AUTHOR_EMAIL,
        description=DESCRIPTION,
        license='MIT',
        long_description=LONG_DESCRIPTION,
        name=read_package_variable('__project__'),
        packages=find_packages(),
        version=read_package_variable('__version__'),
        url=URL,)
